Package: rawDiag
Type: Package
Title: Brings Orbitrap Mass Spectrometry Data to Life; Fast and
        Colorful
Version: 1.9.0
Authors@R: c(person(given = "Christian", family = "Panse",
      email = "cp@fgcz.ethz.ch", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0003-1975-3064")),
	person(given = "Christian", family = "Trachsel",
	  email = "Christian.Trachsel@fgcz.ethz.ch", role = "aut"),
	person(given = "Tobias", family = "Kockmann",
	  email = "Tobias.Kockmann@fgcz.ethz.ch", role = "aut"))
Depends: R (>= 4.4)
Imports: dplyr, ggplot2 (>= 3.4), grDevices, hexbin, htmltools,
        BiocManager, BiocParallel, rawrr (>= 1.15.5), rlang, reshape2,
        scales, shiny (>= 1.5), stats, utils
Suggests: BiocStyle (>= 2.28), ExperimentHub, tartare, knitr, testthat
Description: Optimizing methods for liquid chromatography coupled to mass
  spectrometry (LC-MS) poses a nontrivial challenge. The rawDiag package
  facilitates rational method optimization by generating MS operator-tailored
  diagnostic plots of scan-level metadata. The package is designed for use on
  the R shell or as a Shiny application on the Orbitrap instrument PC.
License: GPL-3
URL: https://github.com/fgcz/rawDiag/
BugReports: https://github.com/fgcz/rawDiag/issues
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.2
VignetteBuilder: knitr
biocViews: MassSpectrometry, Proteomics, Metabolomics, Infrastructure,
        Software, ShinyApps
git_url: https://git.bioconductor.org/packages/rawDiag
git_branch: devel
git_last_commit: d2d26ca
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-29 00:33:29 UTC; biocbuild
Author: Christian Panse [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1975-3064>),
  Christian Trachsel [aut],
  Tobias Kockmann [aut]
Maintainer: Christian Panse <cp@fgcz.ethz.ch>
